package ch.std.book.integration;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.test.context.ActiveProfiles;

import ch.std.book.jpa.Book;

@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
@ActiveProfiles("unittest")
public class BookControllerTests {

	Logger logger = LoggerFactory.getLogger(BookControllerTests.class);

	@Autowired
	private TestRestTemplate restTemplate;	
	private List<Book> bookList = new ArrayList<>();

	@BeforeEach
	public void setup() {
		logger.info("BookControllerTests.setup");
		this.bookList.clear();
		// create test book
		String postUrl = "/rest/book";
		HttpEntity<Book> request = new HttpEntity<>(new Book("1234567890", "test", "test", "test"));
		logger.info("BookControllerTests.setup, postUrl = " + postUrl);
		Book testBook = this.restTemplate.postForObject(postUrl, request, Book.class);
		logger.info("BookControllerTests.setup, testBook = " + testBook);
		this.bookList.add(testBook);
	}

	@Test
	public void testGetBooks() throws Exception {
		String url = "/rest/books?value=test";
		Book[] books = this.restTemplate.getForObject(url, Book[].class);
		assertNotNull(books);
		assertEquals(1, books.length);
		Book book = books[0];
		assertNotNull(book);
		assertEquals("1234567890", book.getIsbn());
	}

	@Test
	public void testGetAllBooks() throws Exception {
		String url = "/rest/books";
		Book[] books = this.restTemplate.getForObject(url, Book[].class);
		assertNotNull(books);
		assertEquals(this.bookList.size(), books.length);
		for (Book book : this.bookList) {
			assertTrue(bookList.contains(book));
		}
	}

	@Test
	public void testGetBookById() throws Exception {
		String url = "/rest/books?value=test";
		Book[] books = this.restTemplate.getForObject(url, Book[].class);
		assertNotNull(books);
		assertEquals(1, books.length);
		Book book = books[0];
		String urlById = "/rest/book/" + book.getId();
		Book bookById = this.restTemplate.getForObject(urlById, Book.class);
		assertEquals(book, bookById);
	}

}
